@@ -3,7 +3,6 @@ import logging
 from datetime import datetime
 
 from django.db.backends.utils import strip_quotes
-from django.db.models import Index
 from django.db.transaction import TransactionManagementError, atomic
 from django.utils import six, timezone
 from django.utils.encoding import force_bytes
@@ -565,16 +564,8 @@ class BaseDatabaseSchemaEditor(object):
         # True               | False            | True               | True
         if old_field.db_index and not old_field.unique and (not new_field.db_index or new_field.unique):
             # Find the index for this field
-            meta_index_names = {index.name for index in model._meta.indexes}
-            # Retrieve only BTREE indexes since this is what's created with
-            # db_index=True.
-            index_names = self._constraint_names(model, [old_field.column], index=True, type_=Index.suffix)
+            index_names = self._constraint_names(model, [old_field.column], index=True)
             for index_name in index_names:
-                if index_name in meta_index_names:
-                    # The only way to check if an index was created with
-                    # db_index=True or with Index(['field'], name='foo')
-                    # is to look at its name (refs #28053).
-                    continue
                 self.execute(self._delete_constraint_sql(self.sql_delete_index, model, index_name))
         # Change check constraints?
         if old_db_params['check'] != new_db_params['check'] and old_db_params['check']:
@@ -975,7 +966,7 @@ class BaseDatabaseSchemaEditor(object):
 
     def _constraint_names(self, model, column_names=None, unique=None,
                           primary_key=None, index=None, foreign_key=None,
-                          check=None, type_=None):
+                          check=None):
         """
         Returns all constraint names matching the columns and conditions
         """
@@ -999,7 +990,5 @@ class BaseDatabaseSchemaEditor(object):
                     continue
                 if foreign_key is not None and not infodict['foreign_key']:
                     continue
-                if type_ is not None and infodict['type'] != type_:
-                    continue
                 result.append(name)
         return result
