@@ -87,7 +87,7 @@ class BaseFormSet(object):
     def __nonzero__(self):      # Python 2 compatibility
         return type(self).__bool__(self)
 
-    @property
+    @cached_property
     def management_form(self):
         """Returns the ManagementForm instance for this FormSet."""
         if self.is_bound:
@@ -161,6 +161,10 @@ class BaseFormSet(object):
             'auto_id': self.auto_id,
             'prefix': self.add_prefix(i),
             'error_class': self.error_class,
+            # Don't render the HTML 'required' attribute as it may cause
+            # incorrect validation for extra, optional, and deleted
+            # forms in the formset.
+            'use_required_attribute': False,
         }
         if self.is_bound:
             defaults['data'] = self.data
@@ -195,6 +199,7 @@ class BaseFormSet(object):
             auto_id=self.auto_id,
             prefix=self.add_prefix('__prefix__'),
             empty_permitted=True,
+            use_required_attribute=False,
             **self.get_form_kwargs(None)
         )
         self.add_fields(form, None)
@@ -331,11 +336,15 @@ class BaseFormSet(object):
         """
         self._errors = []
         self._non_form_errors = self.error_class()
+        empty_forms_count = 0
 
         if not self.is_bound:  # Stop further processing.
             return
         for i in range(0, self.total_form_count()):
             form = self.forms[i]
+            if not form.has_changed():
+                empty_forms_count += 1
+
             self._errors.append(form.errors)
         try:
             if (self.validate_max and
@@ -347,7 +356,7 @@ class BaseFormSet(object):
                     code='too_many_forms',
                 )
             if (self.validate_min and
-                    self.total_form_count() - len(self.deleted_forms) < self.min_num):
+                    self.total_form_count() - len(self.deleted_forms) - empty_forms_count < self.min_num):
                 raise ValidationError(ungettext(
                     "Please submit %d or more forms.",
                     "Please submit %d or more forms.", self.min_num) % self.min_num,
