@@ -12,7 +12,9 @@ import cgi
 import sys
 
 from django.conf import settings
-from django.core.exceptions import SuspiciousMultipartForm
+from django.core.exceptions import (
+    RequestDataTooBig, SuspiciousMultipartForm, TooManyFieldsSent,
+)
 from django.core.files.uploadhandler import (
     SkipFile, StopFutureHandlers, StopUpload,
 )
@@ -35,6 +37,7 @@ class InputStreamExhausted(Exception):
     """
     pass
 
+
 RAW = "raw"
 FILE = "file"
 FIELD = "field"
@@ -58,17 +61,13 @@ class MultiPartParser(object):
         :input_data:
             The raw post data, as a file-like object.
         :upload_handlers:
-            A list of UploadHandler instances that perform operations on the uploaded
-            data.
+            A list of UploadHandler instances that perform operations on the
+            uploaded data.
         :encoding:
             The encoding with which to treat the incoming data.
         """
-
-        #
         # Content-Type should contain multipart and the boundary information.
-        #
-
-        content_type = META.get('HTTP_CONTENT_TYPE', META.get('CONTENT_TYPE', ''))
+        content_type = META.get('CONTENT_TYPE', '')
         if not content_type.startswith('multipart/'):
             raise MultiPartParserError('Invalid Content-Type: %s' % content_type)
 
@@ -81,7 +80,7 @@ class MultiPartParser(object):
         # Content-Length should contain the length of the body we are about
         # to receive.
         try:
-            content_length = int(META.get('HTTP_CONTENT_LENGTH', META.get('CONTENT_LENGTH', 0)))
+            content_length = int(META.get('CONTENT_LENGTH', 0))
         except (ValueError, TypeError):
             content_length = 0
 
@@ -109,9 +108,8 @@ class MultiPartParser(object):
         Parse the POST data and break it into a FILES MultiValueDict and a POST
         MultiValueDict.
 
-        Returns a tuple containing the POST and FILES dictionary, respectively.
+        Return a tuple containing the POST and FILES dictionary, respectively.
         """
-        # We have to import QueryDict down here to avoid a circular import.
         from django.http import QueryDict
 
         encoding = self._encoding
@@ -120,22 +118,24 @@ class MultiPartParser(object):
         # HTTP spec says that Content-Length >= 0 is valid
         # handling content-length == 0 before continuing
         if self._content_length == 0:
-            return QueryDict('', encoding=self._encoding), MultiValueDict()
+            return QueryDict(encoding=self._encoding), MultiValueDict()
 
         # See if any of the handlers take care of the parsing.
         # This allows overriding everything if need be.
         for handler in handlers:
-            result = handler.handle_raw_input(self._input_data,
-                                              self._meta,
-                                              self._content_length,
-                                              self._boundary,
-                                              encoding)
+            result = handler.handle_raw_input(
+                self._input_data,
+                self._meta,
+                self._content_length,
+                self._boundary,
+                encoding,
+            )
             # Check to see if it was handled
             if result is not None:
                 return result[0], result[1]
 
         # Create the data structures to be used later.
-        self._post = QueryDict('', mutable=True)
+        self._post = QueryDict(mutable=True)
         self._files = MultiValueDict()
 
         # Instantiate the parser and stream:
@@ -145,6 +145,13 @@ class MultiPartParser(object):
         old_field_name = None
         counters = [0] * len(handlers)
 
+        # Number of bytes that have been read.
+        num_bytes_read = 0
+        # To count the number of keys in the request.
+        num_post_keys = 0
+        # To limit the amount of data read from the request.
+        read_size = None
+
         try:
             for item_type, meta_data, field_stream in Parser(stream, self._boundary):
                 if old_field_name:
@@ -166,18 +173,39 @@ class MultiPartParser(object):
                 field_name = force_text(field_name, encoding, errors='replace')
 
                 if item_type == FIELD:
+                    # Avoid storing more than DATA_UPLOAD_MAX_NUMBER_FIELDS.
+                    num_post_keys += 1
+                    if (settings.DATA_UPLOAD_MAX_NUMBER_FIELDS is not None and
+                            settings.DATA_UPLOAD_MAX_NUMBER_FIELDS < num_post_keys):
+                        raise TooManyFieldsSent(
+                            'The number of GET/POST parameters exceeded '
+                            'settings.DATA_UPLOAD_MAX_NUMBER_FIELDS.'
+                        )
+
+                    # Avoid reading more than DATA_UPLOAD_MAX_MEMORY_SIZE.
+                    if settings.DATA_UPLOAD_MAX_MEMORY_SIZE is not None:
+                        read_size = settings.DATA_UPLOAD_MAX_MEMORY_SIZE - num_bytes_read
+
                     # This is a post field, we can just set it in the post
                     if transfer_encoding == 'base64':
-                        raw_data = field_stream.read()
+                        raw_data = field_stream.read(size=read_size)
+                        num_bytes_read += len(raw_data)
                         try:
                             data = base64.b64decode(raw_data)
                         except _BASE64_DECODE_ERROR:
                             data = raw_data
                     else:
-                        data = field_stream.read()
+                        data = field_stream.read(size=read_size)
+                        num_bytes_read += len(data)
 
-                    self._post.appendlist(field_name,
-                                          force_text(data, encoding, errors='replace'))
+                    # Add two here to make the check consistent with the
+                    # x-www-form-urlencoded check that includes '&='.
+                    num_bytes_read += len(field_name) + 2
+                    if (settings.DATA_UPLOAD_MAX_MEMORY_SIZE is not None and
+                            num_bytes_read > settings.DATA_UPLOAD_MAX_MEMORY_SIZE):
+                        raise RequestDataTooBig('Request body exceeded settings.DATA_UPLOAD_MAX_MEMORY_SIZE.')
+
+                    self._post.appendlist(field_name, force_text(data, encoding, errors='replace'))
                 elif item_type == FILE:
                     # This is a file, use the handler...
                     file_name = disposition.get('filename')
@@ -200,9 +228,10 @@ class MultiPartParser(object):
                     try:
                         for handler in handlers:
                             try:
-                                handler.new_file(field_name, file_name,
-                                                 content_type, content_length,
-                                                 charset, content_type_extra)
+                                handler.new_file(
+                                    field_name, file_name, content_type,
+                                    content_length, charset, content_type_extra,
+                                )
                             except StopFutureHandlers:
                                 break
 
@@ -229,11 +258,11 @@ class MultiPartParser(object):
 
                             for i, handler in enumerate(handlers):
                                 chunk_length = len(chunk)
-                                chunk = handler.receive_data_chunk(chunk,
-                                                                   counters[i])
+                                chunk = handler.receive_data_chunk(chunk, counters[i])
                                 counters[i] += chunk_length
                                 if chunk is None:
-                                    # If the chunk received by the handler is None, then don't continue.
+                                    # Don't continue if the chunk received by
+                                    # the handler is None.
                                     break
 
                     except SkipFile:
@@ -260,6 +289,7 @@ class MultiPartParser(object):
             if retval:
                 break
 
+        self._post._mutable = False
         return self._post, self._files
 
     def handle_file_complete(self, old_field_name, counters):
@@ -270,9 +300,7 @@ class MultiPartParser(object):
             file_obj = handler.file_complete(counters[i])
             if file_obj:
                 # If it returns a file object, then set the files dict.
-                self._files.appendlist(
-                    force_text(old_field_name, self._encoding, errors='replace'),
-                    file_obj)
+                self._files.appendlist(force_text(old_field_name, self._encoding, errors='replace'), file_obj)
                 break
 
     def IE_sanitize(self, filename):
@@ -392,8 +420,10 @@ class LazyStream(six.Iterator):
         maliciously-malformed MIME request.
         """
         self._unget_history = [num_bytes] + self._unget_history[:49]
-        number_equal = len([current_number for current_number in self._unget_history
-                            if current_number == num_bytes])
+        number_equal = len([
+            current_number for current_number in self._unget_history
+            if current_number == num_bytes
+        ])
 
         if number_equal > 40:
             raise SuspiciousMultipartForm(
@@ -543,20 +573,12 @@ class BoundaryIter(six.Iterator):
 
 
 def exhaust(stream_or_iterable):
-    """
-    Completely exhausts an iterator or stream.
-
-    Raise a MultiPartParserError if the argument is not a stream or an iterable.
-    """
-    iterator = None
+    """Exhaust an iterator or stream."""
     try:
         iterator = iter(stream_or_iterable)
     except TypeError:
         iterator = ChunkIter(stream_or_iterable, 16384)
 
-    if iterator is None:
-        raise MultiPartParserError('multipartparser.exhaust() was passed a non-iterable or stream parameter')
-
     for __ in iterator:
         pass
 
@@ -633,9 +655,11 @@ class Parser(object):
 
 
 def parse_header(line):
-    """ Parse the header into a key-value.
-        Input (line): bytes, output: unicode for key/name, bytes for value which
-        will be decoded later
+    """
+    Parse the header into a key-value.
+
+    Input (line): bytes, output: unicode for key/name, bytes for value which
+    will be decoded later.
     """
     plist = _parse_header_params(b';' + line)
     key = plist.pop(0).lower().decode('ascii')
